<?php

namespace Drupal\evanced_events_importer\Controller;

/**
 * Define help functions.
 */
class Help {

  // Define variable to track HTML block elements.
  private $blockElement = FALSE;

  // Define offset header index ( value must be 0 - 4).
  private $headerOffset = 2;

  /**
   * Get help content from README file.
   *
   * @return string
   *   Text from the help the README file.
   */
  public function getHelpContent() {

    // Define location of README file.
    $file_path = dirname(__FILE__) . '/../../README.md';

    // If file exists...
    if (file_exists($file_path)) {

      // Open README.md.
      $file = fopen($file_path, 'r');

      // Read file content.
      $file_content = fread($file, 25000);

      // Add HTML breaks to new file content.
      $file_text = nl2br($file_content);

      // Return file text.
      return $file_text;

    }
    // Else, README.md file has been deleted.
    else {

      return 'Missing README.md file. Please download from Drupal.';

    }

  }

  /**
   * Logic to return proper header.
   *
   * @param int $level
   *   The level of the header to retrieve.
   *
   * @return string
   *   Element name for the given header level.
   */
  private function getHeaderElement($level) {

    // Return header, as determined by the header offset.
    if ($level + $this->headerOffset == 1) {
      return 'h1';
    }
    elseif ($level + $this->headerOffset == 2) {
      return 'h2';
    }
    elseif ($level + $this->headerOffset == 3) {
      return 'h3';
    }
    elseif ($level + $this->headerOffset == 4) {
      return 'h4';
    }
    elseif ($level + $this->headerOffset == 5) {
      return 'h5';
    }
    elseif ($level + $this->headerOffset >= 6) {
      return 'h6';
    }

  }

  /**
   * Format inline content.
   *
   * @param string $line
   *   The text to format.
   *
   * @return string
   *   The formatted text.
   */
  private function formatInlineContent($line) {

    // Convert escaped characters into entities.
    $line = preg_replace("/\\\\\*/", "&#42;", $line);
    $line = preg_replace("/\\\_/", "&#95;", $line);
    $line = preg_replace("/\\\`/", "&#96;", $line);

    // Format links.
    $line = preg_replace('/\[(.*)\]\((.*)\)/', '<a href="$2">$1</a>', $line);

    // Format bold text.
    $line = preg_replace('/\*\*(.*)\*\*/', '<strong>$1</strong>', $line);

    // Format emphasized text.
    $line = preg_replace('/\*(.*)\*/', '<em>$1</em>', $line);
    $line = preg_replace('/_(.*)_/', '<em>$1</em>', $line);

    // Format code tags.
    $line = preg_replace('/`(.*)`/', '<code>$1</code>', $line);

    return $line;

  }

  /**
   * Format header content.
   *
   * @param string $line
   *   The text to format.
   * @param int $level
   *   The level of the header to retrieve.
   *
   * @return string
   *   The formatted text.
   */
  private function formatHeader($line, $level) {

    $text = '';

    $header = $this->getHeaderElement($level);

    // If the current line is empty...
    if (trim($line) == '') {

      // Close the block.
      $text .= '</' . $header . '>';

    }
    // Else, the line has content.
    else {

      // If a block element is not already in progress...
      if ($this->blockElement === FALSE) {

        // Start a new header.
        $text .= '<' . $header . '>';
        $this->blockElement = 'h' . $level;

      }

      // If the line ends in 2 spaces...
      if (preg_match('/[ ]{2}$/', $line) > 0) {

        // Add the line to the header with a break.
        $text .= preg_replace('/^#+ /', '', trim($line)) . '<br />';

      }
      else {

        // Add the line to the header and close it.
        $text .= preg_replace('/^#+ /', '', trim($line)) . '</' . $header . '>';
        $this->blockElement = FALSE;

      }

    }

    return $text;

  }

  /**
   * Format paragraph content.
   *
   * @param string $line
   *   The text to format.
   * @param string $next
   *   The line of text following the line to be formatted.
   *
   * @return string
   *   The formatted text.
   */
  private function formatParagraph($line, $next) {

    $text = '';

    // If the current line is empty...
    if (trim($line) == '') {

      // Close the block.
      $text .= '</p>';
      $this->blockElement = FALSE;

    }
    // Else, the line has content.
    else {

      // If a block element is not already in progress...
      if ($this->blockElement === FALSE) {

        // Start a new paragraph.
        $text .= '<p>';
        $this->blockElement = 'p';

      }

      // If the line ends in 2 spaces...
      if (preg_match('/[ ]{2}$/', $line) > 0) {

        // Add the line to the header with a break.
        $text .= $line . '<br />';

      }
      else {

        // If the next line is empty.
        if (trim($next) == '') {

          // Add the line to the paragraph and close it.
          $text .= $line . '</p>';
          $this->blockElement = FALSE;

        }
        else {

          // Add the line to the paragraph without closing it.
          $text .= $line;

        }

      }

    }

    return $text;

  }

  /**
   * Format unordered list item content.
   *
   * @param string $line
   *   The text to format.
   * @param string $next
   *   The line of text following the line to be formatted.
   *
   * @return string
   *   The formatted text.
   */
  private function formatUnorderedListItem($line, $next) {

    $text = '';

    // If the current line is empty...
    if (trim($line) == '') {

      // If a list item is open.
      if (strpos($this->blockElement, 'li') !== FALSE) {

        // Close the li block.
        $text .= '</li>';

      }

      // Close the ul block.
      $text .= '</ul>';
      $this->blockElement = FALSE;

    }
    // Else, the line has content.
    else {

      // If a ul element is not already in progress...
      if ($this->blockElement === FALSE) {

        // Start a new unordered list.
        $text .= '<ul>';
        $this->blockElement = 'ul';

      }

      // If an li element is not already in progress...
      if (strpos($this->blockElement, 'li') === FALSE) {

        // Start a new list item.
        $text .= '<li>';
        $this->blockElement = 'ul/li';

      }

      // If the line ends in 2 spaces...
      if (preg_match('/[ ]{2}$/', $line) > 0) {

        // Add the line to the header with a break.
        $text .= preg_replace('/(^- )|(^\* )|(^\+ )/', '', trim($line)) . '<br />';

      }
      else {

        // If the next line is empty.
        if (trim($next) == '') {

          // Add the line to the list item and close it.
          $text .= preg_replace('/(^- )|(^\* )|(^\+ )/', '', trim($line)) . '</li></ul>';
          $this->blockElement = FALSE;

        }
        else {

          // If the next line is a new list item.
          if (preg_match('/(^- )|(^\* )|(^\+ )/', trim($next)) > 0) {

            // Add this line to the current list item and close it.
            $text .= preg_replace('/(^- )|(^\* )|(^\+ )/', '', trim($line)) . '</li>';
            $this->blockElement = 'ul';

          }
          else {

            // Add this line to the current list item without closing it.
            $text .= preg_replace('/(^- )|(^\* )|(^\+ )/', '', trim($line));

          }

        }

      }

    }

    return $text;

  }

  /**
   * Format ordered list item content.
   *
   * @param string $line
   *   The text to format.
   * @param string $next
   *   The line of text following the line to be formatted.
   *
   * @return string
   *   The formatted text.
   */
  private function formatOrderedListItem($line, $next) {

    $text = '';

    // If the current line is empty...
    if (trim($line) == '') {

      // If a list item is open.
      if (strpos($this->blockElement, 'li') !== FALSE) {

        // Close the li block.
        $text .= '</li>';

      }

      // Close the ol block.
      $text .= '</ol>';
      $this->blockElement = FALSE;

    }
    // Else, the line has content.
    else {

      // If a ol element is not already in progress...
      if ($this->blockElement === FALSE) {

        // Start a new ordered list.
        $text .= '<ol>';
        $this->blockElement = 'ol';

      }

      // If an li element is not already in progress...
      if (strpos($this->blockElement, 'li') === FALSE) {

        // Start a new list item.
        $text .= '<li>';
        $this->blockElement = 'ol/li';

      }

      // If the line ends in 2 spaces...
      if (preg_match('/[ ]{2}$/', $line) > 0) {

        // Add the line to the header with a break.
        $text .= preg_replace('/^[0-9]*\. /', '', trim($line)) . '<br />';

      }
      else {

        // If the next line is empty.
        if (trim($next) == '') {

          // Add the line to the list item and close it.
          $text .= preg_replace('/^[0-9]*\. /', '', trim($line)) . '</li></ol>';
          $this->blockElement = FALSE;

        }
        else {

          // If the next line is a new list item.
          if (preg_match('/^[0-9]*\. /', trim($next)) > 0) {

            // Add this line to the current list item and close it.
            $text .= preg_replace('/^[0-9]*\. /', '', trim($line)) . '</li>';
            $this->blockElement = 'ol';

          }
          else {

            // Add this line to the current list item without closing it.
            $text .= preg_replace('/^[0-9]*\. /', '', trim($line));

          }

        }

      }

    }

    return $text;

  }

  /**
   * Formats MarkDown text.
   *
   * @param string $markdownText
   *   The text to format.
   *
   * @return string
   *   The formatted text.
   */
  private function formatMarkdownText($markdownText) {

    // Split file text into an array.
    $file_text_array = explode('<br />', $markdownText);

    // Declare variable to store formatted text.
    $text = '';

    // Loop through each line in the file.
    foreach ($file_text_array as $line_index => $line) {

      // Define variables to hold the previous and next lines.
      $previous = $next = '';

      // If a line preceded  this one...
      if (array_key_exists(($line_index - 1), $file_text_array)) {

        // Store the value of the previous line.
        $previous = $file_text_array[$line_index - 1];

      }

      // If a line follows this one...
      if (array_key_exists(($line_index + 1), $file_text_array)) {

        // Store the value of the next line.
        $next = $file_text_array[$line_index + 1];

      }

      // If line is not a MarkDown header (=== or ---) or horizontal rule...
      if (
        preg_match("/^==*$/", trim($line)) === 0 &&
        preg_match("/^--*$/", trim($line)) === 0
      ) {

        // If this line has content or a block element is in progress...
        if (strlen(trim($line)) > 0 || $this->blockElement !== FALSE) {

          // If the current or next line indicates a header or
          // a header is already in progress...
          if (
            preg_match("/^# /", trim($line)) > 0 ||
            preg_match("/^==*$/", trim($next)) > 0 ||
            $this->blockElement == 'h1'
          ) {

            // Format header.
            $format_result = $this->formatHeader($line, 1);

            // Update local scope with results.
            $text .= $this->formatInlineContent($format_result);

          }
          // Else, if the current or next line indicates a subheader or
          // a subheader is already in progress...
          elseif (
            preg_match("/^## /", trim($line)) > 0 ||
            preg_match("/^--*$/", trim($next)) > 0 ||
            $this->blockElement == 'h2'
          ) {

            // Format header.
            $format_result = $this->formatHeader($line, 2);

            // Update local scope with results.
            $text .= $this->formatInlineContent($format_result);

          }
          // Else, if the current line indicates an h3-h6
          // an h3-h6 is already in progress...
          elseif (
            preg_match("/^###+ /", trim($line)) > 0 ||
            preg_match("/^h[3-6]/", $this->blockElement) > 0
          ) {

            // If a header is already in progress...
            if (preg_match("/^h[3-6]/", $this->blockElement) > 0) {

              // Retrieve the header level from the block element in progress.
              $header_level = preg_replace('/^h/', '', $this->blockElement);

            }
            else {

              // Retrieve the leading hash characters from the current line.
              preg_match("/^###+ /", trim($line), $leading_hashes);

              // Determine header level.
              $header_level = strlen(trim($leading_hashes[0]));

            }

            // Format header.
            $format_result = $this->formatHeader($line, $header_level);

            // Update local scope with results.
            $text .= $this->formatInlineContent($format_result);

          }
          // Else, if the line indicates a ul/li or
          // an unordred list is already in progress...
          elseif (
            preg_match('/(^- )|(^\* )|(^\+ )/', trim($line)) > 0 ||
            $this->blockElement == 'ul/li' ||
            $this->blockElement == 'ul'
          ) {

            // Format header.
            $format_result = $this->formatUnorderedListItem($line, $next);

            // Update local scope with results.
            $text .= $this->formatInlineContent($format_result);

          }
          // Else, if the line indicates an ol/li or
          // an ordered list is already in progress...
          elseif (
            preg_match('/^[0-9]*\. /', trim($line)) > 0 ||
            $this->blockElement == 'ol/li' ||
            $this->blockElement == 'ol'
          ) {

            // Format header.
            $format_result = $this->formatOrderedListItem($line, $next);

            // Update local scope with results.
            $text .= $this->formatInlineContent($format_result);

          }
          // Else, if the line begins with an HTML tag...
          elseif (preg_match('/^<[A-Za-z\/]*>/', trim($line)) > 0) {

            // Add the line with only inline formatting.
            $text .= $this->formatInlineContent($line);

          }
          // Else, format the text as a paragraph.
          else {

            // Format header.
            $format_result = $this->formatParagraph($line, $next);

            // Update local scope with results.
            $text .= $this->formatInlineContent($format_result);

          }

        }

      }
      // Else...
      else {

        // If both the previous line and next line are empty...
        if (trim($next) == '' && trim($previous) == '') {

          // Add a horizontal rule.
          $text .= '<hr />';

        }

      }

    }

    // Return formatted markdown content.
    return $text;

  }

  /**
   * Displays help content.
   *
   * @return array
   *   Render array containing formatted text.
   */
  public function displayHelpContent() {

    return [
      '#type' => 'markup',
      '#markup' => $this->formatMarkdownText($this->getHelpContent()),
    ];

  }

}
