<?php

namespace Drupal\evanced_events_importer\Controller;

use Drupal\Node\Entity\Node;
use Drupal\Taxonomy\Entity\Term;

/**
 * Define Evanced Events Importer functions.
 */
class Importer {

  /**
   * Converts commonly-used Windows-1252 glyphs into alpha character codes.
   *
   * The EXML feed is encoded in ISO-8859-1 format. It's similar to
   * Windows-1252 format, except for a few extra commonly-used
   * glyphs. These glyphs won't display in Drupal unless they're
   * converted into alpha character codes first.
   *
   * @param string $text_string
   *   Text that potentially contains Windows-1252 glyphs to convert.
   */
  private static function convertGlyphCharacters($text_string) {

    // Create an array of the most common Windows-1252 character entities:
    // Specifically, numeric character codes and alpha character codes.
    $glyphs_array = array(
      "&#133;" => "&hellip;",
      "&#145;" => "&lsquo;",
      "&#146;" => "&rsquo;",
      "&#147;" => "&ldquo;",
      "&#148;" => "&rdquo;",
      "&#149;" => "&bull;",
      "&#150;" => "&ndash;",
      "&#151;" => "&mdash;",
      "&#152;" => "&tilde;",
      "&#153;" => "&trade;",
    );

    // Convert glyphs in $text_string into HTML special character entities.
    $text_string = mb_convert_encoding($text_string, "HTML-ENTITIES", "UTF-8");

    // Replace any Windows-1252 numeric character entities (&#148;)
    // with alpha character entities instead (&rdquo;).
    $text_string = strtr($text_string, $glyphs_array);

    return $text_string;

  }

  /**
   * Gets items from Evanced EXML feed and returns them as an object.
   *
   * @param string $feed_url
   *   URL of EXML feed.
   */
  private static function getXmlItems($feed_url) {

    // Initialize the XML parser (Document Object Model (DOM) library).
    $doc = new \DOMDocument();

    // Load the XML file.
    $doc->load(trim($feed_url));

    // Get each 'item' XML element.
    $items_object = $doc->getElementsByTagName('item');

    // Return $items_object_object.
    return $items_object;

  }

  /**
   * Gets attributes from Evanced EXML feed and returns them as an array.
   *
   * @param string $feed_url
   *   URL of EXML feed.
   */
  public static function getItemAttributes($feed_url) {

    $item_attributes_string = '';

    // Retrieve XML items.
    $items_object = self::getXMLItems($feed_url);

    // Retrieve all attributes for this item.
    $item_attributes = $items_object->item(0)->getElementsByTagName("*");

    // Loop through attributes object to turn it into a standard array.
    foreach ($item_attributes as $item_attribute) {

      // Ensure a valid XML tag name exists before continuing.
      if (trim($item_attribute->nodeName) != '') {

        // Append a variable with the tag name of the current element.
        $item_attributes_string .= $item_attribute->nodeName . ',';

      }

    }

    // Remove the last occurrence of a comma in the string.
    $item_attributes_string = rtrim(trim($item_attributes_string), ',');

    // Split the string of tag names into an
    // array, using the comma as a delimiter.
    $item_attributes_array = explode(',', $item_attributes_string);

    return $item_attributes_array;

  }

  /**
   * Converts XML items object into an array for easier processing.
   *
   * @param string $feed_url
   *   URL of EXML feed.
   */
  private static function getItemsArray($feed_url) {

    // Retrieve an object of XML items.
    $items_object = self::getXMLItems($feed_url);

    // Retrieve an array of item attributes.
    $item_attributes_array = self::getItemAttributes($feed_url);

    // Define an index to track current item.
    $item_index = 0;

    // Loop through all XML items to create an easy to use array.
    foreach ($items_object as $item) {

      // Add item to items_array.
      $items_array[$item_index] = [];

      // Loop through item attributes array.
      foreach ($item_attributes_array as $item_attributes_key => $item_attributes_value) {

        // Retrieve data associated with this XML element.
        $item_attribute = $item->getElementsByTagName($item_attributes_value);

        // Add attribute to event in items_array.
        $items_array[$item_index][$item_attribute->item(0)->nodeName] = $item_attribute->item(0)->nodeValue;

      }

      // Increment events count.
      $item_index++;

    }

    return $items_array;

  }

  /**
   * Gets date range of events returned by the XML feed.
   *
   * @param array $items_array
   *   Array of XML items.
   */
  private static function getDateRange($items_array) {

    // Set the default date range. Make sure it's extreme enough
    // to get immediately overwritten by dates in the EXML feed.
    // Thu, 01 Jan 1970 00:00:00 GMT.
    $date_range['max'] = 0;
    // Tue, 19 Jan 2038 03:14:07 GMT.
    $date_range['min'] = 2147483647;

    // Loop through items array.
    foreach ($items_array as $item) {

      // Get date variable.
      $date = strtotime($item['date']);

      // If event date takes place after the maximum date...
      if ($date > $date_range['max']) {
        // Update maximum date variable.
        $date_range['max'] = $date;
      }

      // If event date takes place after the minimum date...
      if ($date < $date_range['min']) {
        // Update maximum date variable.
        $date_range['min'] = $date;
      }

    }

    return $date_range;

  }

  /**
   * Gets array of taxonomy terms and synonyms.
   */
  private static function getTaxonomyArray() {

    // Get taxonomy terms IDs.
    $taxonomy_term_id_array = \Drupal::entityQuery('taxonomy_term')->execute();

    // Define taxonomy array.
    $taxonomy_array = [];

    // Loop through vocabulary IDs.
    foreach ($taxonomy_term_id_array as $taxonomy_term_id) {

      // Load taxonomy object.
      // $taxonomy_object = \Drupal\taxonomy\
      // Entity\Term::load($taxonomy_term_id);.
      $taxonomy_object = Term::load($taxonomy_term_id);

      // Add $taxonomy_term_id to $taxonomy_array.
      $taxonomy_array[$taxonomy_term_id] = [];

      // Store the label for this taxonomy term.
      $taxonomy_array[$taxonomy_term_id][] = $taxonomy_object->getName();

      // Loop through taxonomy object.
      foreach ($taxonomy_object->toArray() as $taxonomy_attribute_key => $taxonomy_attribute_values) {

        // If this attribute key contains the word 'synonym'.
        if (strpos($taxonomy_attribute_key, 'synonym') !== FALSE) {

          // If synonym has value.
          if (count($taxonomy_attribute_values) > 0) {

            // Loop through attribute values.
            foreach ($taxonomy_attribute_values as $taxonomy_attribute_value) {

              // Store the synonym for this taxonomy term.
              $taxonomy_array[$taxonomy_term_id][] = $taxonomy_attribute_value['value'];

            }

          }

        }

      }

    }

    return $taxonomy_array;

  }

  /**
   * Inserts new entity with only minimal content.
   *
   * @param object $item
   *   XML item to be saved as a new entity.
   * @param array $config
   *   Configuration settings.
   * @param array $taxonomy_array
   *   Array of vocabulary terms to match against XML values.
   */
  private static function insertEntity($item, $config, $taxonomy_array) {

    // Define array to store bare minimum values to create an entity.
    $entity_array = array(
      'type' => $config['content_type'],
      'title' => $item['id'],
      'uid' => $config['publisher_uid'],
      'status' => 0,
      $config['entity_id_field'] => $item['id'],
    );

    // Insert a new node.
    $entity = Node::create($entity_array);
    $entity->save();

    // Load newly created entity.
    $entity = Node::load($entity->id());

    // Update entity with complete details.
    self::updateEntity($entity, $item, $config, $taxonomy_array, TRUE);

  }

  /**
   * Updates an existing entity with values from an XML item.
   *
   * @param object $entity
   *   Drupal entity to update.
   * @param array $item
   *   XML item to be used to update the entity.
   * @param array $config
   *   Configuration settings.
   * @param array $taxonomy_array
   *   Array of vocabulary terms to match against XML values.
   * @param array $force_update
   *   Boolean to force an entity update regardless of whether or not
   *   the associated feed item updated within the last 24 hours.
   */
  private static function updateEntity($entity, $item, $config, $taxonomy_array, $force_update = FALSE) {

    // If item was updated within the last 26 hours or force_update is true.
    if (strtotime($item['lastupdated']) != $entity->changed->value || $force_update == TRUE) {

      // If configuration is set to log events...
      if ($config['log_events'] == 1) {
        // Log delete.
        \Drupal::logger('Evanced Events Importer')->notice('Update Event: ' . $item['id']);
      }

      // Get fields for this entity.
      $entity_fields = \Drupal::entityManager()->getFieldMap('node', $config['content_type'])['node'];

      // Loop through field_map.
      foreach ($config['field_map'] as $field => $attributes_array) {

        // Define array to hold entity targets.
        $target_entity_array = [];

        // Loop through attributes_array.
        foreach ($attributes_array as $attribute) {

          // If field is an entity reference.
          if ($entity_fields[$field]['type'] == 'entity_reference') {

            // Loop through $taxonomy_array keys.
            foreach ($taxonomy_array as $taxonomy_term_key => $taxonomy_term_value) {

              // Loop through taxonomy term values.
              foreach ($taxonomy_term_value as $taxonomy_term_synonym) {

                // If the synonym matches the given attribute.
                if ($taxonomy_term_synonym == $item[$attribute]) {

                  // Add value to target entity array.
                  $target_entity_array[]['target_id'] = $taxonomy_term_key;

                  // Update field value.
                  $entity->$field = $target_entity_array;

                }

              }

            }

          }
          // Else if field is an address.
          elseif ($entity_fields[$field]['type'] == 'address') {

            // Split address information into an array.
            $address_array = str_replace('<br>', '<br />', $item[$attribute]);
            $address_array = explode('<br />', $item[$attribute]);

            // If address_array contains only 1 item...
            if (count($address_array) == 1) {

              // Update field values (Location Name).
              $entity->$field->recipient = $address_array[0];

            }
            // Else, render the individual items as separate address parts.
            else {

              // Update field values (Country Code).
              $entity->$field->country_code = 'US';

              // If the first array item begins with a letter,
              // assume it is an address recipient.
              if (ctype_alpha(substr($address_array[0], 0, 1))) {

                // Update field value (recipient).
                $entity->$field->recipient = $address_array[0];

                // Remove recipient name from the array.
                unset($address_array[0]);

                // Update array index.
                $address_array = array_values($address_array);

              }

              // If address_array has 3 items remaining, assume
              // the street address has been split into 2 lines.
              if (count($address_array) == 3) {

                // Update field value (address_line_2).
                $entity->$field->address_line2 = $address_array[1];

                // Remove address_line_2 from the array.
                unset($address_array[1]);

                // Update array index.
                $address_array = array_values($address_array);

              }

              // Update field value (address_line_1).
              $entity->$field->address_line1 = $address_array[0];

              // Split the remaining item into "City, State ZIP".
              $address_details = explode(' ', $address_array[1]);

              // Update field values (City, State, ZIP).
              $entity->$field->locality = trim(str_replace(',', '', $address_details[0]));
              $entity->$field->administrative_area = $entity->$field->country_code . '-' . trim(str_replace(',', '', $address_details[1]));
              $entity->$field->postal_code = trim(str_replace(',', '', $address_details[2]));

            }

          }
          // Else if field is a "datetime".
          elseif ($entity_fields[$field]['type'] == 'datetime') {

            // Note datetime XML attributes that are dates only.
            $date_only_fields_array = array(
              'date',
              'date1',
              'enddate',
              'enddate1',
            );

            // Note datetime XML attributes that are times only.
            $time_only_fields_array = array('time', 'endtime');

            // If attribute has a value...
            if ($item[$attribute] !== '') {

              // If attribute is a date only field...
              if (is_numeric(array_search($attribute, $date_only_fields_array))) {

                // Format datetime. Then update the field value.
                $entity->$field = gmdate('Y-m-d', strtotime($item[$attribute]));

              }
              // Else if attribute is a time only field...
              elseif (is_numeric(array_search($attribute, $time_only_fields_array))) {

                // If this is a start time (time).
                if ($attribute == 'time') {

                  // Format datetime by concatenating
                  // the event date with the time.
                  $item[$attribute] = gmdate('Y-m-d\TH:i:s', strtotime($item['date1'] . ' ' . $item[$attribute]));

                }
                // Else, this is an end time (endtime)
                else {

                  // If an enddate is provided.
                  if ($item['enddate1'] == '') {

                    // Format datetime by concatenating
                    // the event date with the endtime.
                    $item[$attribute] = gmdate('Y-m-d\TH:i:s', strtotime($item['date1'] . ' ' . $item[$attribute]));

                  }
                  else {

                    // Format datetime by concatenating
                    // the event enddate with the endtime.
                    $item[$attribute] = gmdate('Y-m-d\TH:i:s', strtotime($item['enddate1'] . ' ' . $item[$attribute]));

                  }

                }

                // Update the field value.
                $entity->$field = $item[$attribute];

              }
              // Else...
              else {

                // Format datetime. Then update the field value.
                $entity->$field = gmdate('Y-m-d\TH:i:s', strtotime($item[$attribute]));

              }

            }

          }
          // Else, treat this as a generic single-value field.
          else {

            // Update field value.
            $entity->$field = $item[$attribute];

          }

        }

      }

      // Make entity status active.
      $entity->status = 1;

      // Update author.
      $entity->uid = $config['publisher_uid'];

      // If user configuration is set to strip links from the event description.
      if ($config['striplinks_flag'] == 1) {

        // Strip links.
        $entity->body->value = preg_replace('/<a[^>]*?href=[\'"](.*?)[\'"][^>]*?>(.*?)<\/a>/si', '$2', $entity->body->value);

      }
      else {

        // If domain was specified on the configuration page...
        if (trim($config['relative_links_domain']) != '') {

          // Convert links in body field.
          $entity->body->value = preg_replace(
            '/href=\"(http|https):\/\/([a-zA-Z]*.)?' . trim($config['relative_links_domain']) . '\//i',
            'href="' . base_path(),
            $entity->body->value);

        }

      }

      // Convert glyph characters in body field.
      $entity->body->value = self::convertGlyphCharacters($entity->body->value);

      // Render body by configured format.
      $entity->body->format = $config['input_format'];

      // Update timestamp.
      $entity->changed = strtotime($item['lastupdated']);

      // Save changes.
      $entity->save();

    }

  }

  /**
   * Initial kickoff funtion that updates Drupal entities with XML values.
   */
  public static function updateEvents() {

    // Get configuration settings.
    $config = Configuration::getConfigurationSettings();

    // If an XML feed_url and field_map are defined...
    if (($config['feed_url'] != '') && !empty($config['field_map'])) {

      // Get array of items from XML Feed.
      $items_array = self::getItemsArray($config['feed_url']);

      // Define array of item Evanced IDs for quicker index searching.
      $item_Evanced_IDs_array = array_column($items_array, 'id');

      // Get date range of items returned from XML feed.
      $date_range = self::getDateRange($items_array);

      // Get entity IDs of the given content type that fall within
      // the given date range. Used "DANGEROUS_ACCESS_CHECK_OPT_OUT"
      // so search can run under CRON user.
      $entity_id_array = \Drupal::entityQuery('node')
        ->condition($config['entity_start_field'], array(
          gmdate("Y-m-d ", $date_range['min']),
          gmdate("Y-m-d ", $date_range['max']),
        ), 'BETWEEN')
        ->condition('type', $config['content_type'])
        ->addTag('DANGEROUS_ACCESS_CHECK_OPT_OUT')
        ->execute();

      // Get Taxonomy array.
      $taxonomy_array = self::getTaxonomyArray();

      // Loop through entities (Drupal).
      foreach ($entity_id_array as $entity_id) {

        // Load entity so we can work with its values.
        $entity = node::load($entity_id);

        // If entity has an EvancedID...
        if (count($entity->get($config['entity_id_field'])->getValue()) !== 0) {

          // Note entity ID field value for easy reuse.
          $entity_evanced_id = $entity->get($config['entity_id_field'])->getValue()[0]['value'];

          // If entity is found in the item IDs array.
          if (array_search($entity_evanced_id, $item_Evanced_IDs_array) !== FALSE) {

            // Update the entity.
            self::updateEntity($entity, $items_array[array_search($entity_evanced_id, $item_Evanced_IDs_array)], $config, $taxonomy_array, $config['force_update_flag']);

            // Remove from $items_array.
            unset($items_array[array_search($entity_evanced_id, $item_Evanced_IDs_array)]);

            // Update array index.
            $items_array = array_values($items_array);

            // Update item IDs array.
            $item_Evanced_IDs_array = array_column($items_array, 'id');

          }
          // Else, delete the entity.
          else {

            // If configuration is set to log events...
            if ($config['log_events'] == 1) {
              // Log delete.
              \Drupal::logger('Evanced Events Importer')->notice('Delete Event: ' . $entity_evanced_id);
            }

            // Delete the entity.
            $entity->delete();

          }

        }

      }

      // Loop through remaining items from the XML feed.
      foreach ($items_array as $item) {

        // If configuration is set to log events...
        if ($config['log_events'] == 1) {
          // Log insert.
          \Drupal::logger('Evanced Events Importer')->notice('Insert Event: ' . $item['id']);
        }

        // Insert new entity.
        self::insertEntity($item, $config, $taxonomy_array);

      }

    }
    // Else, note the configuration issue in the log.
    else {

      \Drupal::logger('Evanced Events Importer')->notice('Evanced import halted. Please complete module configuration.');

    }

  }

}
