<?php

namespace Drupal\evanced_events_importer\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\evanced_events_importer\Controller\Configuration;

/**
 * Define Evanced Events Importer Main configuration form.
 */
class MainConfigurationForm extends FormBase {

  /**
   * Return form ID.
   */
  public function getFormId() {
    return 'evanced_events_importer_main_configuration';
  }

  /**
   * Build form.
   *
   * @param array $form
   *   Form data.
   * @param FormStateInterface $form_state
   *   Form state.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Get configuration settings.
    $config = Configuration::getConfigurationSettings();

    // Create array to hold form structure.
    $form = [];

    // Create an array to hold input format options.
    $input_format_options = [];

    // Retrieve a list of input formats and set the data as an array.
    $input_formats_array = filter_formats();

    // Make a loop for each input format in the array.
    foreach ($input_formats_array as $input_format_key => $input_format) {
      // Add the appropriate input format data to the options array.
      $input_format_options[$input_format->get('format')] = $input_format->get('name');
    }

    // Define the form fields for the main configuration screen.
    $form['instructions'] = array(
      '#type' => 'markup',
      '#markup' => '<div id="eei_instructions"> <p>'
      . t(
        'This module imports and syncs data from @external-evanced-events with your Drupal site. The module maps elements from the Evanced EXML feed to fields in a Drupal content type to create, update and delete events in Drupal every time cron runs. For more details, review the @internal-readme file.',
        array(
          '@external-evanced-events' => Link::fromTextAndUrl(
            'Evanced Events',
            Url::fromUri('http://evancedsolutions.com/our-solutions/events')
          )->toString(),
          '@internal-readme' => Link::fromTextAndUrl(
            'Read Me',
            Url::fromRoute('evanced_events_importer.help')
          )->toString(),
        )
      )
      . '</p> <p>'
      . 'For help with Evanced Events, please contact your Evanced Solutions representative and/or refer to the Evanced Knowledge Base.'
      . '</p> </div>',
    );

    $form['entity_id_field'] = [
      '#type' => 'hidden',
      '#default_value' => '',
    ];

    $form['entity_start_field'] = [
      '#type' => 'hidden',
      '#default_value' => '',
    ];

    $form['feed_url'] = [
      '#type' => 'textfield',
      '#title' => t('EXML feed URL'),
      '#description' => t('Refer to the Evanced Knowledge Base for available “switches” to customize the feed URL.'),
      '#default_value' => $config['feed_url'],
      '#size' => 35,
      '#maxlength' => 500,
      '#required' => TRUE,
    ];

    $form['content_type'] = [
      '#type' => 'select',
      '#title' => t('Content type'),
      '#options' => node_type_get_names(),
      '#default_value' => $config['content_type'],
      '#description' => t('Content type for mapping EXML feed data.'),
      '#required' => TRUE,
    ];

    $form['publisher_uid'] = [
      '#type' => 'textfield',
      '#title' => t('Event author'),
      '#description' => t('Drupal user ID of the account that will be set as the author of events.'),
      '#default_value' => $config['publisher_uid'],
      '#size' => 4,
      '#required' => TRUE,
    ];

    $form['log_events'] = [
      '#type' => 'checkbox',
      '#title' => t('Log automatic creation, update and deletion of events'),
      '#default_value' => $config['log_events'],
      '#description' => t('Record all changes to events in the Drupal log.'),
    ];

    $form['force_update_flag'] = [
      '#type' => 'checkbox',
      '#title' => t('Force update of existing Drupal events'),
      '#default_value' => $config['force_update_flag'],
      '#description' => t('Overwrites all existing Drupal events with data from the EXML feed. Warning: Your site may experience performance slowdowns with this option enabled. If this option is not enabled, only events with a newer timestamp in the EXML feed will be updated.'),
    ];

    $form['content_options'] = [
      '#type' => 'fieldset',
      '#title' => t('Content Options'),
      '#collapsible' => TRUE,
      '#collapsed' => FALSE,
    ];

    $form['content_options']['input_format'] = [
      '#type' => 'select',
      '#title' => t('Input format'),
      '#options' => $input_format_options,
      '#default_value' => $config['input_format'],
      '#description' => t('Select the input format that will be applied to the main body of this event.'),
    ];

    $form['content_options']['relative_links_domain'] = [
      '#type' => 'textfield',
      '#title' => t('Relative link domain'),
      '#description' => t('Converts absolute links containing the domain above into relative links. (Examples: www.example.com, test.example.com, example.com) If left blank, links will not be converted.'),
      '#default_value' => $config['relative_links_domain'],
      '#size' => 35,
    ];

    $form['content_options']['striplinks_flag'] = [
      '#type' => 'checkbox',
      '#title' => t('Strip all links from the event description'),
      '#default_value' => $config['striplinks_flag'],
      '#description' => t('Removes all &lt;a href&gt; (link) tags from the EXML event description. No other HTML tags will be stripped.'),
    ];

    $form['buttons']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Save configuration'),
    ];

    return $form;

  }

  /**
   * Process form data upon submission.
   *
   * @param array $form
   *   Form data.
   * @param FormStateInterface $form_state
   *   Form state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    // Get form field values from form.
    $form_values = $form_state->getValues();

    // Define form fields array.
    $form_fields_array = array(
      'feed_url',
      'content_type',
      'publisher_uid',
      'input_format',
      'force_update_flag',
      'striplinks_flag',
      'relative_links_domain',
      'log_events',
    );

    // Load config object.
    $config = \Drupal::service('config.factory')->getEditable('evanced_events_importer.settings');

    // Loop through form fields array.
    foreach ($form_fields_array as $field) {
      // Store form field value.
      $config->set($field, $form_values[$field]);
    }

    // Save the configuration.
    $config->save();

    // Ensure that a confirmation message is displayed.
    drupal_set_message(t('The main configuration settings have been saved.'), 'status');
    drupal_set_message(t('Changes to the EXML feed URL or content type may affect mapping. Verify the @internal-xml-mapper settings.', array('@internal-xml-mapper' => Link::fromTextAndUrl('XML Mapper', Url::fromRoute('evanced_events_importer.xml_mapper_form'))->toString())), 'status');

  }

}
