<?php

namespace Drupal\evanced_events_importer\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\evanced_events_importer\Controller\Importer;
use Drupal\evanced_events_importer\Controller\Configuration;

/**
 * Define Evanced Events Importer XML Mapper form.
 */
class XmlMapperForm extends FormBase {

  /**
   * Return form ID.
   */
  public function getFormId() {
    return 'evanced_events_importer_xml_mapper';
  }

  /**
   * Build form.
   *
   * @param array $form
   *   Form data.
   * @param FormStateInterface $form_state
   *   Form state.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {

    // Get configuration settings.
    $config = Configuration::getConfigurationSettings();

    // Create array to hold form structure.
    $form = [];

    // Ensure an XML feed was specified on the previous
    // configuration screen before continuing.
    if ($config['feed_url'] != '') {

      // Attach CSS and JavaScript libraries.
      $form['#attached']['library'][] = 'evanced_events_importer/xml_mapper';

      // Reset the array.
      $select_attributes_array = [];

      // Create an array containing the name
      // of each unique element in the XML feed.
      $xml_attributes_array = Importer::getItemAttributes($config['feed_url']);

      // Make a loop for each XML attribute and populate a new
      // array using the element name as both the array key and the array value.
      foreach ($xml_attributes_array as $key => $element) {
        $select_attributes_array[trim($element)] = trim($element);
      }

      // Loop through fields that have already been mapped.
      foreach ($config['field_map'] as $field) {

        // Loop through XML attributes associated with this field.
        foreach ($field as $xml_attribute) {

          // Remove the current XML attributes from
          // select_attributes_array, since it is already in use.
          unset($select_attributes_array[trim($xml_attribute)]);

        }

      }

      // Define the form fields for the XML mapper screen.
      $form['instructions_div_open'] = [
        '#type' => 'markup',
        '#markup' => '<div id="eei_instructions">',
      ];

      $form['instructions1'] = [
        '#type' => 'markup',
        '#markup' => '<p>' . t('Select an Evanced Events EXML element in the lefthand column. Use the right arrow to map it to the corresponding Drupal content type field in the righthand column. Taxonomy fields can have up to three XML elements mapped to them (i.e., eventtype1, eventtype2, eventtype3).  To remove a mapping, use the left arrow next to the Drupal content type field to return it to the list of Evanced EXML elements.') . '</p>',
      ];

      $form['instructions2'] = [
        '#type' => 'markup',
        '#markup' => '<p><b>'
        . t('Helpful hints:')
        . '</b></p> <ul><li>'
        . t(
          "Review this module's @internal-url-readme file for detailed instructions.", [
            "@internal-url-readme" => Link::fromTextAndUrl(
              'Read Me',
              Url::fromRoute('evanced_events_importer.help')
            )->toString(),
          ]
        )
        . '</li> <li>'
        . t("Be sure to include a field for the Evanced ID in your event content type.")
        . '</li> <li>'
        . t('Be sure to include a field for the event start time that maps to the EXML "time" element.')
        . '</li> <li>'
        . t('To map taxonomy terms, the taxonomy term name in Drupal must exactly match the event type and/or age group names in Evanced Events.')
        . '</li> <li>'
        . t('Review the example mappings below.')
        . ' </li></ul>',
      ];

      $form['instructions_div_close'] = [
        '#type' => 'markup',
        '#markup' => '</div><!-- end-div eei_instructions -->',
      ];

      $form['instructions_additional'] = [
        '#type' => 'details',
        '#title' => t('EXAMPLE MAPPINGS'),
      ];

      $form['instructions_additional']['table'] = [
        '#theme' => 'table',
        '#header' => [
          t('EXML ELEMENT'),
          '',
          t('CONTENT TYPE FIELD'),
          t('Required'),
        ],
        '#rows' => [
          ['title', '=', 'Title', 'X',],
          ['description', '=', 'Body', 'X',],
          ['time', '=', 'Event Start', 'X',],
          ['endtime', '=', 'Event End', '',],
          ['id', '=', 'Evanced ID field', 'X',],
          ['location', '=', 'Taxonomy: Location', '',],
          [
            'agegroup1, agegroup2, agegroup3',
            '=',
            'Taxonomy: Age group',
            '',
          ],
          [
            'eventtype1, eventtype2, eventtype3',
            '=',
            'Taxonomy: Event type',
            '',
          ],
        ],
      ];

      $form['selected_options'] = [
        '#type' => 'hidden',
        '#default_value' => '',
      ];

      $form['div1_open'] = [
        '#type' => 'markup',
        '#markup' => '<div id="xml_tags">',
      ];

      $form['exml_list'] = [
        '#type' => 'select',
        '#multiple' => TRUE,
        '#title' => t('EXML elements'),
        '#options' => array_filter($select_attributes_array),
        '#size' => '25',
        '#default_value' => '',
        '#description' => '',
      ];

      $form['div1_close'] = [
        '#type' => 'markup',
        '#markup' => '</div>',
      ];

      $form['div2_open'] = [
        '#type' => 'markup',
        '#markup' => '<div id="mapped_fields">',
      ];

      foreach (\Drupal::entityManager()->getFieldDefinitions('node', $config['content_type']) as $field_name => $field_definition) {

        // Define variables for reuse.
        $fieldLabel = $field_definition->getLabel();

        // If this is not a base field or it is title.
        if ($field_definition->getFieldStorageDefinition()->isBaseField() === FALSE || $field_name == 'title') {

          $form['content_type_row_' . $field_name . '_open'] = [
            '#type' => 'markup',
            '#markup' => '<div class="content_type_row">',
          ];

          // Create toggle buttons.
          $form['toggle_buttons_' . $field_name] = [
            '#type' => 'markup',
            '#markup' => '<div class="toggle_buttons"><span data-field="'
            . $field_name
            . '" data-action="remove" class="button">&#060;</span><span data-field="'
            . $field_name
            . '" data-action="add" class="button">&#062;</span></div> ',
          ];

          // Reset variables.
          $options_array = [];

          // Loop through fields that may have already been mapped.
          foreach ($config['field_map'] as $mapped_field => $elements) {
            // If the mapped field equals this one...
            if ($mapped_field == $field_name) {
              // Loop through mapped elements.
              foreach ($elements as $element) {
                // Add the current XML element to the options array.
                $options_array[$element] = $element;
              }
            }
          }

          // Set the size of the dropdown menu as a variable.
          if ($field_definition->getType() == 'entity_reference') {
            $field_size = 3;
          }
          else {
            $field_size = 1;
          }

          $form['field_' . $field_name . '_select'] = [
            '#type' => 'select',
            '#multiple' => TRUE,
            '#title' => $fieldLabel,
            '#options' => $options_array,
            '#size' => $field_size,
            '#default_value' => [''],
            '#description' => '',
          ];

          $form['field_' . $field_name . '_list'] = [
            '#type' => 'hidden',
            '#default_value' => implode(',', $options_array),
          ];

          $form['content_type_row_' . $field_name . '_close'] = [
            '#type' => 'markup',
            '#markup' => '</div>',
          ];

        }

      }

      $form['content_type'] = [
        '#type' => 'hidden',
        '#default_value' => $config['content_type'],
      ];

      $form['div2_close'] = [
        '#type' => 'markup',
        '#markup' => '</div>',
      ];

      $form['buttons']['submit'] = [
        '#type' => 'submit',
        '#value' => t('Save configuration'),
      ];

    }
    // Otherwise, no XML feed has been specified yet.
    else {

      // Display a message, instructing the user to enter an XML feed first.
      drupal_set_message(t('To map, enter the @internal-main-configuration URL that comes with your Evanced Events installation. To learn more about the EXML feed, please contact your Evanced representative and/or refer to the Events Manual.', array('@internal-main-configuration' => Link::fromTextAndUrl('EXML feed', Url::fromRoute('evanced_events_importer.main_configuration_form'))->toString())), 'warning');

    }

    return $form;

  }

  /**
   * Process form data upon submission.
   *
   * @param array $form
   *   Form data.
   * @param FormStateInterface $form_state
   *   Form state.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    $form_values = $form_state->getValues();

    // Create field_map variable.
    $field_map = [];

    // Loop through fields.
    foreach (\Drupal::entityManager()->getFieldDefinitions('node', $form_values['content_type']) as $field_name => $field_definition) {

      // If this is not a base field or it is title.
      if (($field_definition->getFieldStorageDefinition()->isBaseField() === FALSE) || $field_name == 'title') {

        // Define variables for reuse.
        $fieldLabel = $field_definition->getLabel();
        $formFieldName = 'field_' . $field_name . '_list';

        // If field has a value....
        if (strlen($form_values[$formFieldName]) !== 0) {

          // Add field and its values to the $fieldmap.
          $field_map[$field_name] = explode(',', $form_values[$formFieldName]);

        }

      }

    }

    // Get stored configuration settings.
    $config = \Drupal::service('config.factory')->getEditable('evanced_events_importer.settings');

    // Store the configuration.
    $config->set('field_map', serialize($field_map));
    $config->save();

    // Ensure that a confirmation message is displayed.
    drupal_set_message(t('The XML Mapper configuration settings have been saved.'), 'status');
    drupal_set_message(t('These settings will be applied the next time cron runs.'), 'status');

  }

}
